<?php

require_once(dirname(__FILE__).'/../conf.php');

function start_debug() {
	global $debug_fd;

	if (!isset($debug_fd))
		$debug_fd = fopen(PROJECTS_DIR . 'debug', 'w');
}

function append_debug() {
	global $debug_fd;

	if (!isset($debug_fd))
		$debug_fd = fopen(PROJECTS_DIR . 'debug', 'a');
}

function debug_fd() {
	global $debug_fd;
	return $debug_fd;
}

function stop_debug() {
	global $debug_fd;

	if (isset($debug_fd))
		fclose($debug_fd);
	unset($debug_fd);
}

function wiki_debug($name, $obj = "", $hsc = false) {
	if (is_scalar($obj)) {
		if ($hsc)
			$obj = htmlspecialchars($obj);
		echo "<p>" . "$name = $obj", "</p>";
	}
	else {
		$obj = print_r($obj, true);
		if ($hsc)
			$obj = htmlspecialchars($obj);		
		echo "<pre> $name = ", $obj, "</pre>";
	}
}

// recursively delete all the files
function delete_dir($dir) {
	if (($dh = opendir($dir)) != false) {
		while (($file = readdir($dh)) !== false) {
    		if ($file === '.' || $file === '..') continue;
    		$file = $dir . $file;
			if (is_dir($file))
				delete_dir($file . '/');
			else
				unlink($file);
		}
		closedir($dh);
	}
	rmdir($dir);
}

function html_color_to_RGB($color) {
	if (strlen($color) == 7 && substr($color, 0, 1) != '#') 
		return array(0, 0, 0);
	$R = hexdec(substr($color, 1, 2));
	$G = hexdec(substr($color, 3, 2));
	$B = hexdec(substr($color, 5, 2));
	return array('R' => $R, 'G' => $G, 'B' => $B);
}

function has_extension($name, $ext) {
	$n = strlen($ext);
	$l = strlen($name);
	$tail = substr($name, $l - $n);
	return stristr($tail, $ext) != false;
}

function is_pdf($name) {
	return has_extension($name, ".pdf");
}

function is_image($name) {
	return has_extension($name, ".png") || 
		has_extension($name, ".jpg") ||
		has_extension($name, ".gif");
}

function html_link_to_file($project, $name) {
	if ($project == NULL) return NULL;
	$id = $project->name() . ":$name";
    return html_wikilink($id);
}

function action_button($button_name, $action = '', $hidden = NULL) {
   	global $ID;
	$form = new Doku_Form('Form_' . $button_name);
	if (!$action) $action = $button_name;
	$form->addHidden('do', $action);
	if (is_array($hidden)) foreach ($hidden as $key => $value)
		$form->addHidden($key, $value);
	$form->addElement(form_makeButton('submit', '', $button_name));
   	return $form->getForm();
}

function button_delete($ID) {
	global $INFO;
	if ($INFO['perm'] < AUTH_DELETE) return '';
   	return action_button('Delete', 'save', array('id' => $ID, 'wikitext' => '', 'summary' => "delete $ID"));
}

function button_remake($ID) {
	global $INFO;
	if ($INFO['perm'] < AUTH_EDIT) return '';
   	return action_button('Remake', 'remake', array('id' => $ID, 'filename' => noNS($ID)));
}

function button_remove($range, $tag) {
	global $INFO;
	if ($INFO['perm'] < AUTH_EDIT) return '';
	global $ID;
	$slices = rawWikiSlices($range, $ID);
   	return action_button('Remove', 'save', array(
		"prefix" => $slices[0] . "\n ",
		"wikitext" => "",
		"suffix" => $slices[2],
		"summary" => 'remove ' . $tag));
}

function button_add($tag, $no_content) {
	global $INFO;
	if ($INFO['perm'] < AUTH_EDIT) return '';
	global $ID;
	$terminate = ($no_content ? '/>' : '></' . $tag . '>');
	$text = rawWiki($ID);
  	return action_button('Add ' . $tag, 'save', array( 
		"prefix" => $text . "\n ",
		'wikitext' => '<' . $tag . ' name=""' . $terminate,
		"suffix" => "",
		"summary" => 'add a <' . $tag . '>'));
}

function button_rename_use($range) {
	global $INFO;
	if ($INFO['perm'] < AUTH_EDIT) return '';
	$project = Project::project();
	if ($project == NULL) return '';
	global $ID;
	$self = noNS($ID);
	$files = array('');
	foreach (array_keys($project->files()) as $file) 
		if ($file != $self) $files[] = $file;
	$form = new Doku_Form("change_use");
	$form->addHidden('do', 'change_use');
	$form->addHidden('range', $range);
	$form->addElement(form_makeMenuField('use',  $files, '', '', '', '', 
		array("onchange" => "submit();")));
	return $form->getForm();
}

function media_path($project_id, $name) {
	$path = implode('/', explode(':', $project_id));
	return DOKU_DATA . 'media/' . $path . '/' . $name; 
}

?>